export const deleteValue = Symbol('deleteValue');
export function applyMutation(conditions, obj, option = {}) {
    const defaultOption = { keepMutation: false };
    Object.assign(defaultOption, option);
    // Create the `deleteValue` symbol
    // Start with a shallow copy of the original object
    let result = { ...obj };
    // Recursively apply the mutation to all properties of the object, regardless of
    // whether or not they have a `mutate` property
    for (const [key, value] of Object.entries(result)) {
        if (value && typeof value == 'object' && Array.isArray(value) == false) {
            result[key] = applyMutation(conditions, value, option);
        }
    }
    // Iterate over the list of conditions and apply the corresponding mutation
    // to the result object
    for (const condition of conditions) {
        let conditionKey;
        let args;
        if (typeof condition != 'string') {
            conditionKey = condition.condition;
            args = condition.args();
        }
        else
            conditionKey = condition;
        const mutation = obj.mutate?.[conditionKey];
        if (mutation) {
            for (const [key, value] of Object.entries(mutation)) {
                if (value === deleteValue) {
                    delete result[key];
                }
                else if (typeof value == 'function') {
                    result[key] = value.bind(this)(result[key], ...(Array.isArray(args) ? args : [args]));
                }
                else {
                    result[key] = value;
                }
            }
        }
    }
    // Remove any remaining `mutate` properties from the result
    if (!option.keepMutation) {
        for (const [key, value] of Object.entries(result)) {
            if (key === 'mutate') {
                delete result[key];
            }
        }
    }
    // Return the result
    return result;
}
